# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-12 18:23
from __future__ import unicode_literals

import djmongo.dataimport.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DataImport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file1', models.FileField(upload_to=djmongo.dataimport.models.update_import_filename, verbose_name=b'File to be imported')),
                ('input_format', models.CharField(choices=[(b'csv', b'Comma Seperated Value (.csv)')], default=b'csv', max_length=4)),
                ('delete_collection_before_import', models.BooleanField(default=False)),
                ('status', models.CharField(default=b'New', editable=False, max_length=10, verbose_name=b'Status')),
                ('response', models.TextField(blank=True, default=b'', max_length=2048, verbose_name=b'Response')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='WriteAPIHTTPAuth',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, max_length=100)),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('json_schema', models.TextField(default=b'{}', help_text=b'Default "{}", means no JSONschema.', max_length=2048, verbose_name=b'JSON Schema')),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
                'verbose_name': 'Write API with HTTPAuth Auth',
                'verbose_name_plural': 'Write APIs with HTTPAuth Auth',
            },
        ),
        migrations.CreateModel(
            name='WriteAPIIP',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, max_length=100)),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('json_schema', models.TextField(default=b'{}', help_text=b'Default "{}", means no JSONschema.', max_length=2048, verbose_name=b'JSON Schema')),
                ('from_ip', models.TextField(default=b'127.0.0.1', help_text=b'Only accept requests from a IP in this list (separated by whitespace). 0.0.0.0 means all.', max_length=2048, verbose_name=b'From IPs')),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
                'verbose_name': 'Write API with IP address Auth',
                'verbose_name_plural': 'Write APIs with IP address Auth',
            },
        ),
        migrations.CreateModel(
            name='WriteAPIoAuth2',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, max_length=100)),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('json_schema', models.TextField(default=b'{}', help_text=b'Default "{}", means no JSONschema.', max_length=2048, verbose_name=b'JSON Schema')),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
                'verbose_name': 'Write API with oAuth2 Auth',
                'verbose_name_plural': 'Write APIs with oAuth2 Auth',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('write', '0004_auto_20160113_1640'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='writeapiip',
            name='groups',
        ),
        migrations.AlterField(
            model_name='writeapihttpauth',
            name='groups',
            field=models.ManyToManyField(help_text='Use ctrl to select multiple groups. If no groups are selected blank, any user may access the API.', related_name='djmongo_write_httpauth_groups', to='auth.Group', blank=True),
        ),
        migrations.AlterField(
            model_name='writeapihttpauth',
            name='slug',
            field=models.SlugField(help_text='The slug is the unique part of the URL for your API.', unique=True, max_length=100),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-12 18:22
from __future__ import unicode_literals

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Permission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('permission_name', models.CharField(choices=[(b'db-all', b'All MongoDB'), (b'db-write', b'Write MongoDB'), (b'db-read', b'Read MongoDB'), (b'create-other-users', b'create-other-users'), (b'create-any-socialgraph', b'create-any-socialgraph'), (b'delete-any-socialgraph', b'delete-any-socialgraph')], max_length=50)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='SocialGraph',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_on', models.DateField(default=datetime.date.today)),
                ('grantee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='grantee', to=settings.AUTH_USER_MODEL)),
                ('grantor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='grantor', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-created_on',),
                'get_latest_by': 'created_on',
            },
        ),
        migrations.AlterUniqueTogether(
            name='socialgraph',
            unique_together=set([('grantor', 'grantee')]),
        ),
        migrations.AlterUniqueTogether(
            name='permission',
            unique_together=set([('user', 'permission_name')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-28 16:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('search', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PublicReadAPI',
            fields=[
                ('id',
                 models.AutoField(
                     auto_created=True,
                     primary_key=True,
                     serialize=False,
                     verbose_name='ID')),
                ('database_name',
                 models.CharField(
                     max_length=256)),
                ('collection_name',
                 models.CharField(
                     max_length=256)),
                ('search_keys',
                 models.TextField(
                     blank=True,
                     default=b'',
                     help_text=b'The default, blank, returns\n                                                all keys. Providing a list of\n                                                keys, separated by whitespace,\n                                                limits the API search to only\n                                                these keys.',
                     max_length=4096)),
            ],
            options={
                'verbose_name': 'Search API using No Auth (Public)',
                'verbose_name_plural': 'Search APIs using No Auth (Public)',
            },
        ),
        migrations.AlterModelOptions(
            name='databaseaccesscontrol',
            options={
                'verbose_name': 'Search API using HTTPAuth',
                'verbose_name_plural': 'Search APIs using HTTPAuth'},
        ),
        migrations.AlterField(
            model_name='databaseaccesscontrol',
            name='search_keys',
            field=models.TextField(
                blank=True,
                default=b'',
                help_text=b'The default, blank, returns\n                                                all keys. Providing a list of\n                                                keys, separated by whitespace,\n                                                limits the API search to only\n                                                these keys.',
                max_length=4096),
        ),
        migrations.AlterField(
            model_name='savedsearch',
            name='return_keys',
            field=models.TextField(
                blank=True,
                default=b'',
                help_text='Default is blank which returns all keys. Separate keys by white space to limitthe keys that are returned.',
                max_length=2048),
        ),
        migrations.AlterField(
            model_name='savedsearch',
            name='type_mapper',
            field=models.TextField(
                default=b'{}',
                max_length=2048,
                verbose_name='Map non-string variables to numbers or Boolean'),
        ),
        migrations.AlterUniqueTogether(
            name='publicreadapi',
            unique_together=set(
                [
                    ('database_name',
                     'collection_name')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-28 16:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('console', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='createhistory',
            name='history',
            field=models.BooleanField(default=False, help_text='Check this to create a historical collection.\n                    When items are updated, the old data is saved in\n                    another collection'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-10-06 14:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('search', '0003_auto_20161006_1437'),
    ]

    operations = [
        migrations.AlterField(
            model_name='httpauthreadapi',
            name='slug',
            field=models.SlugField(
                help_text=b'Give your API a unique name',
                max_length=100,
                unique=True),
        ),
        migrations.AlterField(
            model_name='publicreadapi',
            name='slug',
            field=models.SlugField(
                help_text=b'Give your API a unique name',
                max_length=100,
                unique=True),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.conf import settings
import json
import sys
import csv
from datetime import datetime, date, time
from bson.code import Code
from bson.objectid import ObjectId
from bson import json_util
from pymongo import MongoClient, DESCENDING
from collections import OrderedDict


def run_aggregation_pipeline(database_name, collection_name, pipeline):
    result = False
    mc = MongoClient(host=settings.MONGO_HOST,
                     port=settings.MONGO_PORT)
    db = mc[str(database_name)]
    collection = db[str(collection_name)]

    # explain = db.command('aggregate', collection, pipeline=pipeline, explain=True)
    # print explain
    collection.aggregate(pipeline)

    # print agg_result
    result = True
    return result


def to_json(results_dict):
    return json.dumps(results_dict, indent=4, default=json_util.default)


def normalize_results(results_dict):
    # Define some dummy/default values
    mydt = datetime.now()
    myd = date.today()
    myt = time(0, 0)

    for r in results_dict['results']:
        for k, v in r.items():
            if isinstance(r[k], type(mydt)) or \
               isinstance(r[k], type(myd)) or \
               isinstance(r[k], type(myt)):
                r[k] = v.__str__()
                # print r[k]
    return results_dict


def normalize_list(results_list):
    # Define some dummy/default values
    mydt = datetime.now()
    myd = date.today()
    myt = time(0, 0)
    for r in results_list:
        for k, v in r.items():
            if isinstance(r[k], type(mydt)):
                r[k] = v.__str__()
    return results_list


def query_mongo(
        database_name,
        collection_name,
        query={},
        include_num_results="0",
        skip=0,
        sort=None,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        cast_strings_to_integers=False,
        return_keys=()):
    """return a response_dict  with a list of search results"""

    l = []
    response_dict = {}

    try:
        mc = MongoClient(host=settings.MONGO_HOST,
                         port=settings.MONGO_PORT,
                         document_class=OrderedDict)

        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        # Cast the query to integers
        if cast_strings_to_integers:
            query = cast_number_strings_to_integers(query)

        # print query
        if return_keys:
            return_dict = {}
            for k in return_keys:
                return_dict[k] = 1
            # print "returndict=",return_dict
            mysearchresult = collection.find(
                query, return_dict).skip(skip).limit(limit)
        else:
            mysearchresult = collection.find(query).skip(skip).limit(limit)

        if sort:
            mysearchresult.sort(sort)

        response_dict['code'] = 200
        if include_num_results == "1":
            response_dict['num_results'] = response_dict['num_results'] = int(
                mysearchresult.count(with_limit_and_skip=False))

        if include_num_results == "2":
            response_dict['num_results'] = response_dict['num_results'] = int(
                mysearchresult.count(with_limit_and_skip=True))

        response_dict['type'] = "search-results"
        for d in mysearchresult:
            d['id'] = d['_id'].__str__()
            del d['_id']
            l.append(d)
        response_dict['results'] = l

    except:
        print("Error reading from Mongo")
        print(str(sys.exc_info()))
        response_dict['num_results'] = 0
        response_dict['code'] = 500
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())

    return response_dict


def query_mongo_sort_decend(
        database_name,
        collection_name,
        query={},
        skip=0,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
    return_keys=(),
        sortkey=None):
    """return a response_dict  with a list of search results in decending
    order based on a sort key
    """

    l = []
    response_dict = {}

    try:
        mc = MongoClient(host=settings.MONGO_HOST,
                         port=settings.MONGO_PORT,
                         document_class=OrderedDict)

        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        if return_keys:
            return_dict = {}
            for k in return_keys:
                return_dict[k] = 1
            # print "returndict=",return_dict
            mysearchresult = collection.find(
                query, return_dict).skip(skip).limit(limit).sort(
                sortkey, DESCENDING)
        else:
            mysearchresult = collection.find(query).skip(
                skip).limit(limit).sort(sortkey, DESCENDING)

        # response_dict['num_results']=int(mysearchresult.count(with_limit_and_skip=False))
        response_dict['code'] = 200
        response_dict['type'] = "search-results"
        for d in mysearchresult:
            d['id'] = d['_id'].__str__()
            del d['_id']
            l.append(d)
        response_dict['results'] = l

    except:
        print("Error reading from Mongo")
        print(str(sys.exc_info()))
        response_dict['num_results'] = 0
        response_dict['code'] = 500
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def delete_mongo(database_name, collection_name,
                 query={}, just_one=False):
    """delete from mongo helper"""

    l = []
    response_dict = {}

    try:
        mc = MongoClient(host=settings.MONGO_HOST,
                         port=settings.MONGO_PORT)
        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        mysearchresult = collection.remove(query, just_one)

        response_dict['code'] = 200
        response_dict['type'] = "remove-confirmation"

    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['num_results'] = 0
        response_dict['code'] = 500
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def write_mongo(document, database_name,
                collection_name, update=False):
    """Write a document to the collection. Return a response_dict containing
    the written record. Method functions as both insert or update based on update
    parameter"""

    l = []
    response_dict = {}
    try:
        mc = MongoClient(host=settings.MONGO_HOST, port=settings.MONGO_PORT)
        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        # Cast the query to integers
        # if settings.CAST_ININGS_TO_INTEGERS:
        #    query = cast_number_strings_to_integers(query)

        potential_key_found = False
        existing_transaction_id = None
        existing_mongo_id = None

        # enforce non-repudiation constraint on create
        # if document.has_key("transaction_id"):
        #    existing_transaction_id = collection.find_one({'transaction_id':document['transaction_id']})
        #    if existing_transaction_id:
        #        potential_key_found = True

        if "id" in document:
            document["_id"] = ObjectId(document["id"])
            del document["id"]

        if "_id" in document:
            existing_mongo_id = collection.find_one({'_id': document['_id']})
            if existing_mongo_id:
                potential_key_found = True

        if update == False and potential_key_found == True:
            """409 conflict"""
            response_dict['code'] = 409
            response_dict['type'] = "Error"
            response_dict['results'] = []
            response_dict[
                'message'] = "Perhaps you meant to perform an update instead?"
            response_dict['errors'] = [
                "Conflict. This transaction_id has already been created.", ]
            return response_dict

        elif update and potential_key_found:  # this is an update
            # set kwargs _id to the existing_id to force to overwrite existing
            # document

            # if existing_transaction_id:
            #
            #    document['_id'] = ObjectId(existing_transaction_id['_id'])
            #    document['history']=True
            #    history_collection_name = "%s_history" % str(collection_name)
            #    history_collection   = db[str(history_collection_name)]
            #
            #    history_object = existing_transaction_id
            #    history_object['historical_id'] = existing_transaction_id['_id']
            #    del history_object['_id']
            #    #now write the record to the historical collection
            #    written_object = history_collection.insert(history_object)

            if existing_mongo_id:
                document['_id'] = ObjectId(existing_mongo_id['_id'])
                document['history'] = True
                document['verified'] = False
                history_collection_name = "%s_history" % str(collection_name)
                history_collection = db[str(history_collection_name)]

                # print history_collection
                # print existing_mongo_id

                history_object = existing_mongo_id

                history_object['historical_id'] = existing_mongo_id['_id']
                del history_object['_id']
                # print history_object

                # now write the record to the historical collection
                written_object = history_collection.insert(history_object)

            # update the record
            myobjectid = collection.save(document)

        else:
            # this is new so perform an insert.
            myobjectid = collection.insert(document)

        # now fetch the record we just wrote so that we write it back to the
        # DB.
        myobject = collection.find_one({'_id': myobjectid})
        response_dict['code'] = 200
        response_dict['type'] = "write-results"
        myobject['id'] = myobject['_id'].__str__()
        del myobject['_id']
        l.append(myobject)
        response_dict['results'] = l

    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def bulk_csv_import_mongo(csvfile, database_name, collection_name,
                          delete_collection_before_import=False):
    """return a response_dict  with a list of search results"""
    """method can be insert or update"""

    l = []
    response_dict = {}
    try:
        mconnection = MongoClient(settings.MONGO_HOST, settings.MONGO_PORT)
        db = mconnection[database_name]
        collection = db[collection_name]

        if delete_collection_before_import:
            myobjectid = collection.remove({})

        # open the csv file.
        csvhandle = csv.reader(open(csvfile._get_path(), 'rb'), delimiter=',')

        rowindex = 0
        errors = 0
        error_list = []
        success = 0
        for row in csvhandle:

            if rowindex == 0:
                column_headers = row
                cleaned_headers = []
                for c in column_headers:
                    c = c.replace(".", "")
                    c = c.replace("$", "-")
                    c = c.replace(" ", "_")
                    cleaned_headers.append(c)
            else:

                record = OrderedDict(zip(cleaned_headers, row))
                # if there is no values, skip the key value pair
                kwargs = OrderedDict()

                # Only populate fields that are not blank.
                for k, v in record.items():
                    if v:
                        if v.isdigit():
                            kwargs[k] = int(v)
                        else:
                            kwargs[k] = v
                try:

                    myobjectid = collection.insert(kwargs)
                    success += 1
                except:
                    error_message = "Error on row " + \
                        rowindex + ". " + str(sys.exc_info())
                    error_list.append(str(sys.exc_info()))

            rowindex += 1

        if error_list:
            response_dict = {}
            response_dict['num_rows_imported'] = rowindex
            response_dict['num_rows_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors"
        else:

            response_dict = {}
            response_dict['num_rows_imported'] = success
            response_dict['code'] = 200
            response_dict['message'] = "Completed."
        return response_dict

    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def build_non_observational_key(k):

    if str(k).__contains__("__"):
        model_field_split = str(k).split("__")
        newlabel = "%s_" % (model_field_split[0])

        field_occurence_split = str(model_field_split[1]).split("_")

        for i in field_occurence_split[:-1]:
            newlabel = "%s_%s" % (newlabel, i)
        return newlabel
    return k


def get_collection_keys(database_name, collection_name):
    l = []
    try:
        mconnection = MongoClient(settings.MONGO_HOST, settings.MONGO_PORT)
        db = mconnection[database_name]
        ckey_collection = "%s_keys" % (collection_name)
        collection = db[ckey_collection]
        result = collection.find({}).distinct("_id")
        for r in result:
            l.append(r)

        if getattr(settings, 'SORTCOLUMNS', False):
            nl = []  # new list list
            # sort the list according to our list

            for i in getattr(settings, 'SORTCOLUMNS', False):
                for j in l:
                    if j.__contains__(i):
                        nl.append(j)
            difflist = list(set(l) - set(nl))

            for i in difflist:
                nl.append(i)
            return nl

        else:
            return sorted(l)
    except:
        print("Error.", str(sys.exc_info()))
        return []


def build_keys_with_mapreduce(database_name, collection_name):
    map = Code("function() { "
               "    for (var key in this)"
               "        { emit(key, null); } }"
               )
    reduce = Code("function(key, stuff)"
                  "{ return null; }"
                  )

    mconnection = MongoClient(settings.MONGO_HOST, settings.MONGO_PORT)
    db = mconnection[database_name]

    collection = db[collection_name]
    result_collection_name = "%s_keys" % (collection_name)

    result = collection.map_reduce(map, reduce, result_collection_name)
    return None


def raw_query_mongo_db(kwargs, database_name, collection_name):
    # for key in kwargs:
    #    print "arg: %s: %s" % (key, kwargs[key])
    """return a result list or an empty list"""
    l = []
    response_dict = {}

    try:
        mconnection = MongoClient(settings.MONGO_HOST,
                                  settings.MONGO_PORT,
                                  document_class=OrderedDict)
        db = mconnection[database_name]
        transactions = db[collection_name]
        mysearchresult = transactions.find(kwargs)
        mysearchcount = mysearchresult.count()
        if mysearchcount > 0:
            response_dict['code'] = 200
            for d in mysearchresult:
                l.append(d)
            response_dict['results'] = l
    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['code'] = 400

        response_dict['type'] = "Error"
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def cast_number_strings_to_integers(d):
    """d is a dict"""
    for k, v in d.items():
        # print type(v)
        if determine_if_str_or_unicode(v):
            if v.isdigit():
                d[k] = int(v)
    return d


def determine_if_str_or_unicode(s):
    # if str or unicode return True, else False.
    if isinstance(s, str) or isinstance(s, unicode):
        return True
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
import json
from django.shortcuts import render
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from .utils import (
    show_dbs,
    mongodb_drop_collection,
    mongodb_drop_database,
    mongo_delete_json_util,
    mongo_create_json_util)
from .forms import (EnsureIndexForm, DeleteForm, DocumentForm,
                    CreateDatabaseForm, ConfirmDropForm)
from bson.objectid import ObjectId
from collections import OrderedDict
from ..write.models import WriteAPIHTTPAuth, WriteAPIIP
from ..search.models import (CustomHTTPAuthReadAPI, CustomPublicReadAPI,
                             HTTPAuthReadAPI, PublicReadAPI)


def show_apis(request, database_name, collection_name):

    # Get all of the Read APIs
    custom_httpauth_read_apis = CustomHTTPAuthReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    custom_public_read_apis = CustomPublicReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)

    simple_public_read_apis = PublicReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    simple_httpauth_read_apis = HTTPAuthReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)

    # Get all of the Write APIs
    ip_write_apis = WriteAPIIP.objects.filter(database_name=database_name,
                                              collection_name=collection_name)
    httpauth_write_apis = WriteAPIHTTPAuth.objects.filter(
        database_name=database_name, collection_name=collection_name)

    context = {"custom_httpauth_read_apis": custom_httpauth_read_apis,
               "custom_public_read_apis": custom_public_read_apis,
               "simple_public_read_apis": simple_public_read_apis,
               "simple_httpauth_read_apis": simple_httpauth_read_apis,
               "httpauth_write_apis": httpauth_write_apis,
               "ip_write_apis": ip_write_apis,
               'database_name': database_name,
               'collection_name': collection_name}
    return render(request, 'djmongo/console/show-apis.html',
                  context)


def showdbs(request):

    dbs = show_dbs()
    # print(json.dumps(dbs, indent=4))
    cleaned_dbs = []
    if not dbs:
        messages.error(
            request,
            _("""Unable to connect to MongoDB.
              Check that it is running and accessible."""))
    else:
        for i in dbs:
            # only keep non-system DBs.
            if i.get('name', '') != 'admin' and i.get('name', '') != 'local':
                cleaned_dbs.append(i)

    if dbs and not cleaned_dbs:
        messages.info(
            request,
            _("""You have no databases to work on. Please create one."""))

    #     #We have data so let's grab all APIs
    #     apis = []
    #
    #     # Get all of the APIs.
    #     for d in cleaned_dbs:
    #         for c in d['collections']:
    #             # databse_name, collection_name, APIType, AuthType, HTTP Method, object
    #             # new row
    #             api = []
    #             # database name 0
    #             database_name = d['name']
    #             api.append(database_name)
    #
    #             # collection name 1
    #             api.append(c)
    #
    #             # API Type 2
    #             api.append('Basic')
    #
    #             # AuthType 3
    #             api.append("None")
    #
    #             # HTTP Method 4
    #             api.append("GET")
    #
    #             # Read Public
    #             read_public_list = PublicReadAPI.objects.filter(database_name = database_name,
    #                                                collection_name = c)
    #
    #
    #             database['apis'].append(api)
    #
    #             # HTTPAuth Read
    #             api = {}
    #             api['read_httpauth'] = HTTPAuthReadAPI.objects.filter(database_name = database_name,
    #                                                collection_name = c)
    #             database['apis'].append(api)
    #
    #             # Write Httpauth
    #             api = {}
    #             api['write_httpauth'] = WriteAPIHTTPAuth.objects.filter(database_name = database_name,
    #                                                collection_name = c)
    #             database['apis'].append(api)
    #
    #             # Write IP Auth
    #             api = {}
    #             api['write_ip_auth'] = WriteAPIIP.objects.filter(database_name = database_name,
    #                                                collection_name = c)
    #             database['apis'].append(api)
    #
    #             # Custom Read http Auth
    #             api = {}
    #             api['custom_read_httpauth'] = SavedSearch.objects.filter(database_name = database_name,
    #                                                collection_name = c)
    #             database['apis'].append(api)                #collection[c].append(api)
    #         apis.append(database)
    #
    # pp = pprint.PrettyPrinter(indent=4)
    # pp.pprint(apis)

    context = {"dbs": cleaned_dbs}
    return render(request, 'djmongo/console/showdbs.html', context)


def drop_collection(request, database_name, collection_name):
    """Drop Collection"""
    name = """Retype "%s" to drop the collection""" % (collection_name)
    if request.method == 'POST':
        form = ConfirmDropForm(request.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            if name != collection_name:
                messages.error(request, _('The name did not match. \
                                         Drop operation aborted'))
                return HttpResponseRedirect(
                    reverse(
                        'djmongo_drop_collection',
                        args=(
                            database_name,
                            collection_name)))

            response = mongodb_drop_collection(database_name, collection_name)
            if response:
                errormsg = _("ERROR", response)
                messages.error(request, errormsg)
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
            else:
                messages.success(request, _("The collection was deleted."))
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})
    # This is a GET
    context = {'name': name,
               'form': ConfirmDropForm(
                   initial={})}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def drop_database(request, database_name):
    name = """Retype "%s" to drop the database""" % (database_name)
    if request.method == 'POST':
        form = ConfirmDropForm(request.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            if name != database_name:
                messages.error(request, _('The name did not match. \
                                         Drop operation aborted.'))
                return HttpResponseRedirect(reverse('show_dbs'))

            response = mongodb_drop_database(database_name)
            if response:
                errormsg = _("ERROR", response)
                messages.error(request, errormsg)
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
            else:
                messages.success(request, _("The database was deleted."))
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})
    # This is a GET
    context = {'name': name,
               'form': ConfirmDropForm(
                   initial={})}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def simple_ensure_index(request, database_name, collection_name):
    """Ensure a MongoDB index on a particular field name"""
    name = "Ensure a MongoDB index on a particular field name"

    if request.method == 'POST':
        form = EnsureIndexForm(request.POST)

        if form.is_valid():
            result = form.save(database_name, collection_name)
            messages.success(request,
                             _("Index for %s created successfully" % result))
            return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    else:
        # this is a GET
        context = {'name': name,
                   'form': EnsureIndexForm(
                       initial={"database_name": database_name,
                                "collection_name": collection_name})
                   }
        return render(request, 'djmongo/console/generic/bootstrapform.html',
                      context)


def create_new_database(request):
    """Create a New Mongo Database by adding a single document."""
    name = "Create a New MongoDB Database"

    if request.method == 'POST':
        form = CreateDatabaseForm(request.POST)

        if form.is_valid():
            result = form.save()
            if "error" in result:
                messages.error(
                    request, "The database creation operation failed.")
                messages.error(request, result["error"])
            else:
                messages.success(request, "Database created.")
            return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name})

    else:
        # this is a GET
        context = {'name': name,
                   'form': CreateDatabaseForm(
                       initial={"initial_document": '{ "foo" : "bar" }'
                                })
                   }
        return render(request, 'djmongo/console/generic/bootstrapform.html',
                      context)


def create_collection(request, database_name):
    """Create a new MongoDB collection by adding a single document."""
    name = "Create a New Collection"

    if request.method == 'POST':
        form = CreateDatabaseForm(request.POST)

        if form.is_valid():
            result = form.save()
            if "error" in result:
                messages.error(
                    request, _("The database creation operation failed."))
                messages.error(request, result["error"])
            else:
                messages.success(request, "Collection created.")
            return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    else:
        # this is a GET
        context = {'name': name,
                   'form': CreateDatabaseForm(
                       initial={"initial_document": '{ "foo" : "bar" }',
                                "database_name": database_name})
                   }
        return render(request, 'djmongo/console/generic/bootstrapform.html',
                      context)


def remove_data_from_collection(request, database_name, collection_name):

    name = _("""Delete select information from a MongoDB Collection
             based on a query""")

    if request.method == 'POST':
        form = DeleteForm(request.POST)
        if form.is_valid():
            query = form.cleaned_data['query']
            just_one = form.cleaned_data['just_one']

            # run the delete
            results = mongo_delete_json_util(
                query,
                database_name=database_name,
                collection_name=collection_name,
                just_one=just_one)

            # convert to json and respond.
            results_json = json.dumps(results, indent=4)
            return HttpResponse(results_json, status=int(results['code']),
                                content_type="application/json")
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': DeleteForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_document_in_collection(request, database_name, collection_name):
    name = _("Create a Document from JSON")

    if request.method == 'POST':
        form = DocumentForm(request.POST)
        if form.is_valid():
            document = form.cleaned_data['document']
            # create the document
            results = mongo_create_json_util(document,
                                             database_name=database_name,
                                             collection_name=collection_name)

            # convert to json and respond.
            results_json = json.dumps(results, indent=4)
            return HttpResponse(results_json,
                                status=int(results['code']),
                                content_type="application/json")
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name})
    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name, 'form': DocumentForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def update_document_in_collection(request, database_name, collection_name):
    name = _("Update a Document from JSON")

    if request.method == 'POST' or request.method == 'PUT':
        form = DocumentForm(request.POST)
        if form.is_valid():
            document = form.cleaned_data['document']
            doc = json.loads(document, object_pairs_hook=OrderedDict)
            if "_id" not in doc and "id" not in doc:
                result = {"code": 400, "type": "Error", "message": _(
                    "Updates must include either id or _id.")}
                results_json = json.dumps(result, indent=4)
                return HttpResponse(results_json, status=result['code'],
                                    content_type="application/json")

            if "_id" in doc and "id" in doc:
                result = {"code": 400, "type": "Error", "message": _(
                    "Updates cannot contain both id and _id")}
                results_json = json.dumps(result, indent=4)
                return HttpResponse(results_json, status=result['code'],
                                    content_type="application/json")

            if "id" in doc:
                doc["_id"] = ObjectId(doc["id"])
                del doc["id"]

            if "_id" in doc:
                doc["_id"] = ObjectId(doc["_id"])

            # run the update
            results = mongo_create_json_util(
                document,
                database_name=database_name,
                collection_name=collection_name,
                update=True)

            # convert to json and respond.
            results_json = json.dumps(results, indent=4)
            return HttpResponse(results_json, status=int(results['code']),
                                content_type="application/json")
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': DocumentForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-28 16:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('write', '0005_auto_20160113_2202'),
    ]

    operations = [
        migrations.AlterField(
            model_name='writeapiip',
            name='from_ip',
            field=models.TextField(default=b'127.0.0.1', help_text='Only accept requests from a IP in this list separated by whitespace . 0.0.0.0 means all.', max_length=2048, verbose_name='From IPs'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.contrib import admin
from .models import CreateHistory


class CreateHistoryAdmin(admin.ModelAdmin):
    list_display = ('database_name', 'collection_name', 'history')
admin.site.register(CreateHistory, CreateHistoryAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0006_require_contenttypes_0002'),
        ('write', '0002_auto_20160113_1634'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='writeapihttpauth',
            name='group',
        ),
        migrations.RemoveField(
            model_name='writeapiip',
            name='group',
        ),
        migrations.RemoveField(
            model_name='writeapioauth2',
            name='group',
        ),
        migrations.AddField(
            model_name='writeapihttpauth',
            name='groups',
            field=models.ManyToManyField(related_name='djmongo_write_httpauth_groups', to='auth.Group', blank=True),
        ),
        migrations.AddField(
            model_name='writeapiip',
            name='groups',
            field=models.ManyToManyField(related_name='djmongo_write_ip_groups', to='auth.Group', blank=True),
        ),
        migrations.AddField(
            model_name='writeapioauth2',
            name='groups',
            field=models.ManyToManyField(related_name='djmongo_write_oauth_groups', to='auth.Group', blank=True),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django import forms
from .models import (HTTPAuthReadAPI, PublicReadAPI,
                     CustomHTTPAuthReadAPI, CustomPublicReadAPI)


class HTTPAuthReadAPIForm(forms.ModelForm):

    class Meta:
        model = HTTPAuthReadAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'groups')
    required_css_class = 'required'


class PublicReadAPIForm(forms.ModelForm):

    class Meta:
        model = PublicReadAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys',)
    required_css_class = 'required'


class CustomHTTPAuthReadAPIForm(forms.ModelForm):

    class Meta:
        model = CustomHTTPAuthReadAPI
        fields = ('slug', 'query', 'group', 'type_mapper',
                  'return_keys', 'sort', 'default_limit',
                  'database_name', 'collection_name', 'output_format')

    required_css_class = 'required'


class CustomPublicReadAPIForm(forms.ModelForm):

    class Meta:
        model = CustomPublicReadAPI
        fields = ('slug', 'query', 'type_mapper',
                  'return_keys', 'sort', 'default_limit',
                  'database_name', 'collection_name', 'output_format')

    required_css_class = 'required'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib import admin
from .models import (PublicReadAPI, HTTPAuthReadAPI,
                     CustomHTTPAuthReadAPI, CustomPublicReadAPI)


class CustomHTTPAuthReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'creation_date')
admin.site.register(CustomHTTPAuthReadAPI, CustomHTTPAuthReadAPIAdmin)


class CustomPublicReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'creation_date')
admin.site.register(CustomPublicReadAPI, CustomPublicReadAPIAdmin)


class HTTPAuthReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name')
admin.site.register(HTTPAuthReadAPI, HTTPAuthReadAPIAdmin)


class PublicReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name')
admin.site.register(PublicReadAPI, PublicReadAPIAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-12 18:24
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0007_alter_validators_add_error_messages'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DatabaseAccessControl',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('is_public', models.BooleanField(default=False)),
                ('search_keys', models.TextField(blank=True, default=b'', help_text=b'The default, blank, returns all keys.\n                                        Providing a list of keys, seperated by whitespace,\n                                        limits the API search to only these keys.', max_length=4096)),
                ('groups', models.ManyToManyField(blank=True, related_name='djmongo_database_access_control', to='auth.Group')),
            ],
            options={
                'verbose_name_plural': 'Database Access Controls',
            },
        ),
        migrations.CreateModel(
            name='SavedSearch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('output_format', models.CharField(choices=[(b'json', b'JSON'), (b'html', b'HTML'), (b'csv', b'Comma Seperated Value (.csv)')], default=b'json', max_length=4)),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('query', models.TextField(default=b'{}', max_length=2048, verbose_name=b'JSON Query')),
                ('type_mapper', models.TextField(default=b'{}', max_length=2048, verbose_name=b'Map non-string variables to numbers or boolean')),
                ('is_public', models.BooleanField(default=False, help_text=b'If checked, the search can be run without authentication')),
                ('sort', models.TextField(blank=True, default=b'', help_text=b'e.g. [["somefield", 1], ["someotherfield", -1] ]', max_length=2048, verbose_name=b'Sort Dict')),
                ('return_keys', models.TextField(blank=True, default=b'', help_text=b'Default is blank which returns all keys.\n                            Seperate keys by whitespace to limit the keys that are returned.', max_length=2048)),
                ('default_limit', models.IntegerField(default=200, help_text=b'Limit results to this number unless specified otherwise.')),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('group', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
                'verbose_name_plural': 'Saved Searches',
            },
        ),
        migrations.AlterUniqueTogether(
            name='databaseaccesscontrol',
            unique_together=set([('database_name', 'collection_name')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-10-06 14:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0007_alter_validators_add_error_messages'),
        ('search', '0002_auto_20160928_1628'),
    ]

    operations = [
        migrations.CreateModel(
            name='HTTPAuthReadAPI',
            fields=[
                ('id',
                 models.AutoField(
                     auto_created=True,
                     primary_key=True,
                     serialize=False,
                     verbose_name='ID')),
                ('database_name',
                 models.CharField(
                     max_length=256)),
                ('collection_name',
                 models.CharField(
                     max_length=256)),
                ('slug',
                 models.SlugField(
                     blank=True,
                     help_text=b'Give your API a unique name',
                     max_length=100)),
                ('search_keys',
                 models.TextField(
                     blank=True,
                     default=b'',
                     help_text=b'The default, blank, returns\n                                                all keys. Providing a list of\n                                                keys, separated by whitespace,\n                                                limits the API search to only\n                                                these keys.',
                     max_length=4096)),
                ('groups',
                 models.ManyToManyField(
                     blank=True,
                     related_name='djmongo_http_auth_read_api',
                     to='auth.Group')),
            ],
            options={
                'verbose_name': 'Search API using HTTPAuth',
                'verbose_name_plural': 'Search APIs using HTTPAuth',
            },
        ),
        migrations.AlterUniqueTogether(
            name='databaseaccesscontrol',
            unique_together=set(
                []),
        ),
        migrations.RemoveField(
            model_name='databaseaccesscontrol',
            name='groups',
        ),
        migrations.AddField(
            model_name='publicreadapi',
            name='slug',
            field=models.SlugField(
                blank=True,
                help_text=b'Give your API a unique name',
                max_length=100),
        ),
        migrations.DeleteModel(
            name='DatabaseAccessControl',
        ),
        migrations.AlterUniqueTogether(
            name='httpauthreadapi',
            unique_together=set(
                [
                    ('database_name',
                     'collection_name')]),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.shortcuts import render, get_object_or_404
import json
import sys
from django.contrib import messages
from django.views.decorators.csrf import csrf_exempt
from ..decorators import (httpauth_login_required, ip_verification_required,
                          kickout_400, kickout_404, kickout_500)
from django.http import HttpResponse, HttpResponseRedirect
from collections import OrderedDict
from ..mongoutils import write_mongo
from jsonschema import validate
from django.core.urlresolvers import reverse
from jsonschema.exceptions import ValidationError
from .models import WriteAPIHTTPAuth, WriteAPIIP
from .forms import (WriteAPIHTTPAuthForm, WriteAPIHTTPAuthDeleteForm,
                    WriteAPIIPDeleteForm, WriteAPIIPForm)
from django.utils.translation import ugettext_lazy as _


@csrf_exempt
@httpauth_login_required
def write_to_collection_httpauth(request, slug):
    try:
        wapi = WriteAPIHTTPAuth.objects.get(slug=slug)
    except WriteAPIHTTPAuth.DoesNotExist:
        return kickout_404(
            "The API was not not found. Perhaps you need to define it?")

    # ----------------------------------------------------
    if request.method == 'GET':
        try:
            json_schema = json.loads(
                wapi.json_schema, object_pairs_hook=OrderedDict)
            return HttpResponse(
                json.dumps(
                    json_schema,
                    indent=4),
                content_type="application/json")
        except:
            return kickout_500("The JSON Schema did not contain valid JSON")

    # ----------------------------------------------------
    if request.method == 'POST':

        # Check if request body is JSON ------------------------
        try:
            j = json.loads(request.body, object_pairs_hook=OrderedDict)
            if not isinstance(j, type({})):
                kickout_400(
                    "The request body did not contain a JSON object i.e. {}.")
        except:
            return kickout_400("The request body did not contain valid JSON.")

        # check json_schema is valid
        try:
            json_schema = json.loads(
                wapi.json_schema, object_pairs_hook=OrderedDict)

        except:
            return kickout_500(
                "The JSON Schema on the server did not contain valid JSON")

        # Check jsonschema
        if json_schema:
            try:
                validate(j, json_schema)
            except ValidationError:
                msg = "JSON Schema Conformance Error. %s" % (
                    str(sys.exc_info()[1][0]))
                return kickout_400(msg)
        # write_to_mongo
        response = write_mongo(j, wapi.database_name, wapi.collection_name)
        return HttpResponse(json.dumps(response, indent=4),
                            content_type="application/json")


@csrf_exempt
@ip_verification_required
def write_to_collection_ip_auth(request, slug):

    try:
        wapi = WriteAPIIP.objects.get(slug=slug)
    except WriteAPIIP.DoesNotExist:
        return kickout_404(
            "The API was not not found. Perhaps you need to define it?")

    # ----------------------------------------------------
    if request.method == 'GET':
        try:
            json_schema = json.loads(
                wapi.json_schema, object_pairs_hook=OrderedDict)
            return HttpResponse(
                json.dumps(
                    json_schema,
                    indent=4),
                content_type="application/json")
        except:
            return kickout_500("The JSON Schema did not contain valid JSON")

    # ----------------------------------------------------
    if request.method == 'POST':

        # Check if request body is JSON ------------------------
        try:
            j = json.loads(request.body, object_pairs_hook=OrderedDict)
            if not isinstance(j, type(OrderedDict())):
                kickout_400(
                    "The request body did not contain a JSON object i.e. {}.")
        except:
            return kickout_400("The request body did not contain valid JSON.")

        # check json_schema is valid
        try:
            json_schema = json.loads(
                wapi.json_schema, object_pairs_hook=OrderedDict)

        except:
            return kickout_500(
                "The JSON Schema on the server did not contain valid JSON")

        # Check jsonschema
        if json_schema:
            try:
                validate(j, json_schema)
            except ValidationError:
                msg = "JSON Schema Conformance Error. %s" % (
                    str(sys.exc_info()[1][0]))
                return kickout_400(msg)
        # write_to_mongo
        response = write_mongo(j, wapi.database_name, wapi.collection_name)
        return HttpResponse(json.dumps(response, indent=4),
                            content_type="application/json")


def browse_ip_write_apis(request, database_name=None, collection_name=None):
    """Deprecated"""
    name = "Write APIs Using IP-based Authentication"
    if database_name and collection_name:
        wapis = WriteAPIIP.objects.filter(
            database_name=database_name,
            collection_name=collection_name)
    else:
        wapis = WriteAPIIP.objects.all()
    context = {'name': name, 'wapis': wapis,
               'database_name': database_name,
               'collection_name': collection_name}
    return render(
        request,
        'djmongo/console/browse-ip-write-apis.html',
        context)


def browse_httpauth_write_apis(
        request,
        database_name=None,
        collection_name=None):
    """Deprecated"""
    name = "Write APIs Using HTTPAuth Authentication"
    if database_name and collection_name:
        wapis = WriteAPIHTTPAuth.objects.filter(
            database_name=database_name,
            collection_name=collection_name)
    else:
        wapis = WriteAPIHTTPAuth.objects.all()
    context = {'name': name, 'wapis': wapis,
               'database_name': database_name,
               'collection_name': collection_name}
    return render(
        request,
        'djmongo/console/browse-httpauth-write-apis.html',
        context)


def create_httpauth_write_api(
        request,
        database_name=None,
        collection_name=None):
    name = _("Create an HTTP Auth Write API")
    if request.method == 'POST':
        form = WriteAPIHTTPAuthForm(request.POST)
        if form.is_valid():
            a = form.save(commit=False)
            a.created_by = request.user
            a.save()
            msg = _('The HTTP Auth write API for %s was created.') % (a.slug)
            messages.success(request, msg)

            return HttpResponseRedirect(reverse('djmongo_show_apis',
                                                args=(database_name,
                                                      collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': WriteAPIHTTPAuthForm(initial=idata)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def create_ip_write_api(request, database_name=None, collection_name=None):
    name = _("Create an IP-based Write API")
    if request.method == 'POST':
        form = WriteAPIIPForm(request.POST)
        if form.is_valid():
            a = form.save(commit=False)
            a.created_by = request.user
            a.save()
            msg = _('The IP-based write API for %s was created.') % (a.slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': WriteAPIIPForm(initial=idata)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def edit_httpauth_write_api(request, slug):
    a = get_object_or_404(WriteAPIHTTPAuth, slug=slug)
    name = _("Edit HTTP Auth Write API")
    if request.method == 'POST':
        form = WriteAPIHTTPAuthForm(request.POST, instance=a)
        if form.is_valid():
            a = form.save(commit=False)
            a.created_by = request.user
            a.save()
            msg = _('The HTTP Auth API for %s was updated.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)
    # this is a GET
    context = {'name': name,
               'form': WriteAPIHTTPAuthForm(instance=a)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def edit_ip_write_api(request, slug):
    a = get_object_or_404(WriteAPIHTTPAuth, slug=slug)
    name = _("Edit IP-based Write API")
    if request.method == 'POST':
        form = WriteAPIIPForm(request.POST, instance=a)
        if form.is_valid():
            a = form.save(commit=False)
            a.created_by = request.user
            a.save()
            msg = _('The IP-based API for %s was updated.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    context = {'name': name,
               'form': WriteAPIIPForm(instance=a)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def delete_httpauth_write_api(request, slug):
    name = _("Delete HTTP Auth Write API")
    if request.method == 'POST':
        form = WriteAPIHTTPAuthDeleteForm(request.POST)
        if form.is_valid():
            a = WriteAPIHTTPAuth.objects.get(slug=slug)
            database_name = a.database_name
            collection_name = a.collection_name
            a.delete()
            msg = _('The HTTP Auth API for %s was deleted.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse(
                    'djmongo_browse_httpauth_write_apis_w_params',
                    args=(
                        database_name,
                        collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    context = {'name': name,
               'form': WriteAPIHTTPAuthDeleteForm()
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )


def delete_ip_write_api(request, slug):
    name = _("Delete IP-based Write API")
    if request.method == 'POST':
        form = WriteAPIIPDeleteForm(request.POST)
        if form.is_valid():
            a = WriteAPIIP.objects.get(slug=slug)
            database_name = a.database_name
            collection_name = a.collection_name
            a.delete()
            msg = _('The IP-based API for %s was deleted.') % (slug)
            messages.success(request, msg)
            return HttpResponseRedirect(
                reverse(
                    'djmongo_browse_ip_write_apis_w_params',
                    args=(
                        database_name,
                        collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            context = {'form': form, 'name': name}
            return render(
                request,
                'djmongo/console/generic/bootstrapform.html',
                context)

    # this is a GET
    context = {'name': name,
               'form': WriteAPIIPDeleteForm()
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context,
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('write', '0003_auto_20160113_1638'),
    ]

    operations = [
        migrations.RenameField(
            model_name='writeapihttpauth',
            old_name='user',
            new_name='created_by',
        ),
        migrations.RenameField(
            model_name='writeapiip',
            old_name='user',
            new_name='created_by',
        ),
        migrations.RenameField(
            model_name='writeapioauth2',
            old_name='user',
            new_name='created_by',
        ),
        migrations.AlterField(
            model_name='writeapihttpauth',
            name='json_schema',
            field=models.TextField(default=b'{}', help_text=b'Default "{}", means no JSON Schema.', max_length=2048, verbose_name=b'JSON Schema'),
        ),
        migrations.AlterField(
            model_name='writeapiip',
            name='json_schema',
            field=models.TextField(default=b'{}', help_text=b'Default "{}", means no JSON Schema.', max_length=2048, verbose_name=b'JSON Schema'),
        ),
        migrations.AlterField(
            model_name='writeapioauth2',
            name='json_schema',
            field=models.TextField(default=b'{}', help_text=b'Default "{}", means no JSON Schema.', max_length=2048, verbose_name=b'JSON Schema'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.utils.translation import ugettext_lazy as _
from django import forms
from .models import WriteAPIHTTPAuth, WriteAPIIP


class WriteAPIHTTPAuthForm(forms.ModelForm):

    class Meta:
        model = WriteAPIHTTPAuth
        fields = (
            'slug',
            'database_name',
            'collection_name',
            'groups',
            'json_schema')
    required_css_class = 'required'


class WriteAPIHTTPAuthDeleteForm(forms.Form):

    slug = forms.CharField(label=_("Slug"),
                           help_text=_("Retype the slug to confirm deletion."))

    def clean_slug(self):
        slug = self.cleaned_data["slug"]
        try:
            WriteAPIHTTPAuth.objects.get(slug=slug)
        except WriteAPIHTTPAuth.DoesNotExist:
            raise forms.ValidationError(_('There is no API with that slug.'))


class WriteAPIIPForm(forms.ModelForm):

    class Meta:
        model = WriteAPIIP
        fields = (
            'slug',
            'database_name',
            'collection_name',
            'from_ip',
            'json_schema')
    required_css_class = 'required'


class WriteAPIIPDeleteForm(forms.Form):

    slug = forms.CharField(label=_("Slug"),
                           help_text=_("Retype the slug to confirm deletion."))

    def clean_slug(self):
        slug = self.cleaned_data["slug"]
        try:
            WriteAPIIP.objects.get(slug=slug)
        except WriteAPIIP.DoesNotExist:
            raise forms.ValidationError(_('There is no API with that slug.'))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-28 16:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aggregations', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aggregation',
            name='output_collection_name',
            field=models.CharField(help_text=b'The resulting collection.\n        Do not include $out in your pipeline.', max_length=256),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-12 18:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CreateHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('database_name', models.CharField(max_length=100, unique=True)),
                ('collection_name', models.CharField(max_length=100, unique=True)),
                ('history', models.BooleanField(default=False, help_text=b'Check this to create a historical collection. When items are updated, the old data is saved in another collection')),
            ],
            options={
                'verbose_name_plural': 'Create Histories',
            },
        ),
        migrations.AlterUniqueTogether(
            name='createhistory',
            unique_together=set([('database_name', 'collection_name')]),
        ),
    ]



from django.contrib import admin
from .models import DataImport


class DataImportAdmin(admin.ModelAdmin):
    list_display = ('database_name', 'collection_name',
                    'status', 'creation_date')

admin.site.register(DataImport, DataImportAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class CreateHistory(models.Model):

    database_name = models.CharField(max_length=100, unique=True)
    collection_name = models.CharField(max_length=100, unique=True)
    history = models.BooleanField(
        default=False,
        help_text=_("""Check this to create a historical collection.
                    When items are updated, the old data is saved in
                    another collection"""))

    class Meta:
        verbose_name_plural = _("Create Histories")
        unique_together = (('database_name', 'collection_name'),)

    def __str__(self):
        return "%s/%s" % (self.database_name, self.collection_name)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0006_require_contenttypes_0002'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('write', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='writeapihttpauth',
            options={'ordering': ('-creation_date',), 'get_latest_by': 'creation_date', 'verbose_name': 'Write API with HTTPAuth', 'verbose_name_plural': 'Write APIs with HTTPAuth'},
        ),
        migrations.AlterModelOptions(
            name='writeapiip',
            options={'ordering': ('-creation_date',), 'get_latest_by': 'creation_date', 'verbose_name': 'Write API with IP Address Auth', 'verbose_name_plural': 'Write APIs with IP Address Auth'},
        ),
        migrations.AddField(
            model_name='writeapihttpauth',
            name='group',
            field=models.ForeignKey(blank=True, to='auth.Group', null=True),
        ),
        migrations.AddField(
            model_name='writeapihttpauth',
            name='user',
            field=models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AddField(
            model_name='writeapiip',
            name='group',
            field=models.ForeignKey(blank=True, to='auth.Group', null=True),
        ),
        migrations.AddField(
            model_name='writeapiip',
            name='user',
            field=models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AddField(
            model_name='writeapioauth2',
            name='group',
            field=models.ForeignKey(blank=True, to='auth.Group', null=True),
        ),
        migrations.AddField(
            model_name='writeapioauth2',
            name='user',
            field=models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
    ]
